//=========================================================================
//
//	FILE:				CP_DemoImageView.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View to display images using CP_Graphics.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 7 $
//						Latest Checkout:  $Date: 2005-08-30 20:31:18 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================

	// Precompile header support
#include "CP_Precomp.h"

	// CP_Graphics
#include "CP_DemoImageView.h"

	// CPLAT_Framework
#include "CP_Debugging.h"
#include "CP_Environment.h"
#include "CP_Errors.h"
#include "CP_Exceptions.h"
#include "CP_Graphics.h"
#include "CP_Types.h"

const SInt16 kCP_GraphicsDemo_ImageWidth									= 216;
const SInt16 kCP_GraphicsDemo_ImageHeight									= 144;

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CP_DemoImageView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inSuperview			The CP_View that contains this view.
*	@param		inUpperLeft			Upperleft coordinate of view.
*	@param		inContentSize		The size of the view.
*	@param		inUseOffscreen		Flag to indicate if the image should be
*									drawn using an offscreen canvas.
*	
*	@exception	none
*/
CP_DemoImageView::CP_DemoImageView( CPLAT::CP_View *inSuperview,
	const CPLAT::CP_Point& inUpperLeft, const CPLAT::CP_Size& inContentSize,
	bool inUseOffscreen )
		: CPLAT::CP_View( inSuperview, inUpperLeft, inContentSize, true )
{
	fUseOffscreen = inUseOffscreen;
}

//=========================================================================
//	FUNCTION:	~CP_DemoImageView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*
*	@exception	none
*/
CP_DemoImageView::~CP_DemoImageView() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Drawing ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	OnDraw
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Perform the actual drawing of the view.
*
*	@param		inCanvas			CP_Canvas object to draw into.
*	@param		inUpdateArea		CP_Region containing the area to draw.
*	
*	@exception	none
*
*	@return		void
*/
void CP_DemoImageView::OnDraw( CPLAT::CP_Canvas& inCanvas, CPLAT::CP_Region& /*inUpdateArea*/ )
{
	try
	{
		if( fUseOffscreen )
		{
			CPLAT::OSGraphicsPort			currentPort = inCanvas.GetOSPort();
			CPLAT::CP_Rect					bounds( 0, 0, kCP_GraphicsDemo_ImageWidth, kCP_GraphicsDemo_ImageHeight );
			CPLAT::CP_OffscreenCanvas		offscreen( bounds, inCanvas ); 	
			CPLAT::CP_Graphics				graphicsOne( offscreen );
			CPLAT::CP_Rect					destBounds( 0, 0, kCP_GraphicsDemo_ImageWidth, kCP_GraphicsDemo_ImageHeight );

				// draw image into offscreen canvas
			offscreen.BeginDraw();
				graphicsOne.DrawImage( fImageSpec, destBounds );
			offscreen.EndDraw();
			
			CPLAT::CP_Region				updateRegion( bounds );
			offscreen.CopyImage( currentPort, bounds, updateRegion );
		}
		else
		{
			CPLAT::CP_Graphics			graphicsOne( inCanvas );
			CPLAT::CP_Rect				destBounds;
				
				// normal image
			destBounds.Set( 0, 0, kCP_GraphicsDemo_ImageWidth, kCP_GraphicsDemo_ImageHeight );
			graphicsOne.DrawImage( fImageSpec, destBounds );
			
				// with clipping path
			graphicsOne.SaveState();
			
			destBounds.Set( 0, kCP_GraphicsDemo_ImageHeight + 10, kCP_GraphicsDemo_ImageWidth, (kCP_GraphicsDemo_ImageHeight * 2) + 10 );
			destBounds.Inset( 30, 30 );
			
				// make oval clip path and set in graphics object
			graphicsOne.BeginPath();
			graphicsOne.PathAddOval( destBounds );
			graphicsOne.ClosePath();
			graphicsOne.ClipPath();

				// draw image using the just set clipping path
			graphicsOne.DrawImage( fImageSpec, destBounds );

			graphicsOne.RestoreState();

				// image portion
			CPLAT::CP_Rect		sourceBounds( 0, 0, kCP_GraphicsDemo_ImageWidth, kCP_GraphicsDemo_ImageHeight); //( 30, 30, kCP_GraphicsDemo_ImageWidth - 60, kCP_GraphicsDemo_ImageHeight - 60 );
			sourceBounds.Inset( 30, 30 );
			
			destBounds.Set( 0, 0, kCP_GraphicsDemo_ImageWidth, kCP_GraphicsDemo_ImageHeight );
			destBounds.Offset( kCP_GraphicsDemo_ImageWidth, kCP_GraphicsDemo_ImageHeight );
			graphicsOne.DrawImage( fImageSpec, sourceBounds, destBounds );

				// scaled image - 1/2 size
			destBounds.Set( 0, (kCP_GraphicsDemo_ImageHeight * 2) + 20, (kCP_GraphicsDemo_ImageWidth / 2), ((kCP_GraphicsDemo_ImageHeight * 2) + 20) + (kCP_GraphicsDemo_ImageHeight / 2) );
			graphicsOne.DrawImage( fImageSpec, destBounds );
		}
	}
	catch(...)
	{
	}
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Image ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	SetSpec
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Sets the file system spec used to draw the image.
*
*	@param		inSpec				The file system spec used to draw the image.
*	
*	@exception	none
*
*	@return		void
*/
void CP_DemoImageView::SetSpec( CPLAT::CP_FileSystemSpec& inSpec )
{
	fImageSpec = inSpec;

	Refresh();
}

#if TARGET_OS_MAC

#pragma mark -
#pragma mark === Mac specific ===
#pragma mark -

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#pragma mark -
#pragma mark === Win specific ===
#pragma mark -

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#pragma mark -
#pragma mark === Linux specific ===
#pragma mark -

#endif	// TARGET_OS_LINUX