//=========================================================================
//
//	FILE:				CP_Canvas.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			The CP_Canvas class is a wrapper class for a platforms
//						graphics port used for drawing into.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 256 $
//						Latest Checkout:  $Date: 2005-08-26 19:35:37 -0700 (Fri, 26 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_Canvas
#define _H_CP_Canvas
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FontSpec.h"
#include "CP_RGBColor.h"
#include "CP_Types.h"

	// Standard Library
#include <memory>

CPLAT_Begin_Namespace_CPLAT

	// forward class declaration
class CP_Font;

/*!
*	@class		CP_Canvas
*	@brief		The CP_Canvas class is a wrapper class for a platforms graphics port
*				used for drawing into.
*	
*				All drawing in CPLAT is done by drawing into some type of CP_Canvas object. There
*				is also a CP_Canvas object that can be used for offscreen drawing.
*
*				Note: A CP_Canvas object does not maintain a permanent graphics port, and is
*				only valid during the time that the underlying OS wants to draw some type
*				of information.
*/
class CP_EXPORT CP_Canvas
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Canvas();
										CP_Canvas( OSGraphicsPort inGraphicsPort );
										CP_Canvas( const CP_Canvas& inCanvas );
	//! Destructor.
	virtual								~CP_Canvas();
	//@}

	//! @name Colors
	//@{
			void						SetForeColor( const CP_RGBColor& inForeColor );
			void						SetBackColor( const CP_RGBColor& inBackColor );

			CP_RGBColor					GetForeColor();
			CP_RGBColor					GetBackColor();
	//@}

	//! @name Font
	//@{
			CP_Font						*GetFont();
			void						SetFontSpec( CP_FontSpec& inFontSpec );
	//@}

	//! @name Getters
	//@{
			SInt16						GetBitDepth();
			bool						IsColor();
	//@}

	//! @name OSGraphicsPort
	//@{
										/*!
										*	Returns the platforms native graphics port
										*	
										*	@exception	none
										*	
										*	@return		OSGraphicsPort			A platforms native graphics port.
										*/
			OSGraphicsPort				GetOSPort()										{ return( fOSGraphicsPort ); }
			void						SetOSPort( OSGraphicsPort inGraphicsPort );
	//@}

	//! @name Operators
	//@{
			CP_Canvas&					operator = ( const CP_Canvas& inCanvas );
	//@}

	//! @name Setters
	//@{
			void						SetPenSize( SInt16 inSize );
			void						SetTextMode( SInt16 inMode );
	//@}

	//! @name Testers
	//@{
			bool						IsValid() const;
	//@}
			
protected:

	// class data
protected:
	OSGraphicsPort						fOSGraphicsPort;		/*! A typedefed data member that corresponds to the
																	native graphics port for the OS that CPLAT is running
																	on. It is typedefed in CP_Types.h */

	bool								fIsValid;				/*! Flag to indicate if the canvas is currently valid for drawing into. */

	#if TARGET_OS_MAC
		CGContextRef					fContextRef;			/*! MacOSX Quartz context ref for this port. */
		bool							fHasValidContextRef;	/*! Flag to indicate if the context ref is valid. */
		bool							fOwnsContext;			/*! This class created the context ref, and is responsible for its deletion. */
		bool							fUsingQDCoords;			/*! The context ref is using QuickDraw coordinates. */
	#endif
	
	#if TARGET_OS_WIN32
		CP_RGBColor						fForeColor;				/*! Foreground color of objects. */
		CP_RGBColor						fBackColor;				/*! Background or fill color of objects. */
		
		HPEN							fPen;					/*! Windows HPEN used for drawing lines. */
		HPEN							fSavedPen;				/*! Saved Windows HPEN to restore if we changed pens. */
	#endif
	
	#if TARGET_OS_LINUX
		GtkWidget						*fDestWidget;			/*! The GtkView that is being drawn into. */
		GdkDrawable						*fGdkDrawable;			/*! The GtkDrawable that is being drawn into. */
		CP_RGBColor						fForeColor;				/*! Foreground color of objects. */
		CP_RGBColor						fBackColor;				/*! Background or fill color of objects. */
	#endif
		
	CP_Font								*fFont;					/*! Font object used with this canvas. This object is only
																	created if the user wants to mofidy the canvas font.*/
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.	
										CP_Canvas( CGContextRef inContextRef );
	//@}

	//! @name ContextRef
	//@{
										/*!
										*	Flag to indicate if have set the context ref for this object, and it
										*	is valid to draw into.
										*	
										*	@exception	none
										*	
										*	@return		bool			True if this canvas has a context ref that is valid,
										*								and ready to draw into.
										*/
	virtual	bool						HasValidContextRef()							{ return( fHasValidContextRef ); }

										/*!
										*	Flag to indicate if our content ref is currently using QuickDraw coordinates.
										*	
										*	@exception	none
										*	
										*	@return		bool			True if the context ref is using QuickDraw coordinates.
										*/
			bool						UsingQDCoords()									{ return( fUsingQDCoords ); }

										/*!
										*	Sets the lag to indicate if our content ref is currently using QuickDraw coordinates.
										*	
										*	@param		inUsingQDCoords		True if the context ref is using QuickDraw coordinates.
										*	
										*	@exception	none
										*/
			void						SetUsingQDCoords( bool inUsingQDCoords )		{ fUsingQDCoords = inUsingQDCoords; }

	virtual	CGContextRef				GetContextRef( bool inSyncOriginWithPort = true );

										/*!
										*	Sets the context ref to the passed in one. If the passed
										*	in context ref is not NULL, the canvas object is marked
										*	as having a valid context ref.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetContextRef( CGContextRef inContextRef )		{ fContextRef = inContextRef; if( fContextRef != NULL ) fHasValidContextRef = true; }
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Canvas(
												GtkWidget *inGtkWidget,
												GdkDrawable *inDrawable,
												GdkGC *inGraphicsContext );
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the GdkDrawable used to represent this canvas object.
										*	
										*	@exception	none
										*	
										*	@return		GdkDrawable		The GdkDrawable we are representing.
										*/
			GdkDrawable					*GetDrawable()									{ return( fGdkDrawable ); }

										/*!
										*	The GtkWidget that this canvas object is drawing into.
										*	
										*	@exception	none
										*	
										*	@return		GtkWidget		The GtkWidget we are drawing into.
										*/
			GtkWidget					*GetGtkWidget()									{ return( fDestWidget ); }
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Canvas

