//=========================================================================
//
//	FILE:				CP_DataValueHolder.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Template based class to hold a data value of any type.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_DataValueHolder
#define _H_CP_DataValueHolder
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataValueHolder.h"
#include "CP_Debugging.h"
#include "CP_ErrorManager.h"
#include "CP_Exceptions.h"
#include "CP_M_ReferenceCounted.h"
#include "CP_New.h"
#include "CP_SmartPtr.h"

	// Standard Library
#include <algorithm>
#include <typeinfo>

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_ErrorManager;

/*!
*	@class	CP_DataValue
*	@brief	Base class to represent any type of data.
*/
class CP_DataValue
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										/*!
										*	Constructor.
										*/
			 							CP_DataValue()									{}

	//! Destructor.
										/*!
										*	Destructor.
										*/
	virtual								~CP_DataValue()									{}
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the type_info of this class.
										*
										*	@return	type_info		Returns std::type_info about this data value.
										*/
	virtual const std::type_info&		type() const = 0;
	//@}

	//! @name Copying
	//@{
										/*!
										*	Copy are data value. Need to override this class.
										*
										*	@return	CP_DataValue	Default is to return NULL.
										*/
	virtual CP_DataValue				*Copy() const									{ return( NULL ); }
	//@}

	// class data
private:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

template <typename TDataValue>
class CP_DataValueTemp : public CP_DataValue
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										/*!
										*	Constructor.
										*/
			 							CP_DataValueTemp( const TDataValue& inValue )
			 								: fDataValue( inValue )						{}
	//! Destructor.
										/*!
										*	Destructor.
										*/
	virtual								~CP_DataValueTemp()								{}
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the type_info of this class.
										*
										*	@return	type_info		Returns std::type_info about this data value.
										*/
	virtual const std::type_info&		type() const 									{ return( typeid( fDataValue ) ); }

			const TDataValue&			GetDataValue() const							{ return( fDataValue ); }
	//@}

	//! @name Copying
	//@{
										/*!
										*	Copy are data value.
										*
										*	@return	CP_DataValue	Returns a copy of our data value.
										*/
	virtual CP_DataValue				*Copy() const 									{ return( CP_new CP_DataValueTemp( fDataValue ) ); }
	//@}

	// class data
private:
	TDataValue							fDataValue;		/*! The data value being held by this object. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

/*!
*	@class	CP_DataValueHolder
*	@brief	Class to hold a data value of any type.
*/
class CP_EXPORT CP_DataValueHolder
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										/*!
										*	Constructor.
										*/
										CP_DataValueHolder()
											: fDataValuePtr(NULL)									{}

										#if TARGET_CC_MSVC_6
											CP_DataValueHolder( CP_DataValueHolder& inDataValueHolder )
										#else
											CP_DataValueHolder( const CP_DataValueHolder& inDataValueHolder )
										#endif
											: fDataValuePtr(NULL)
												{
													if( inDataValueHolder.fDataValuePtr != NULL )
														fDataValuePtr = inDataValueHolder.fDataValuePtr->Copy();
												}

										/*!
										*	Constructor from the passed in template value.
										*
										*	@param	inValue		Template value to assign to our data value member.
										*/
					 					template <typename T>
					 					CP_DataValueHolder( const T& inValue )
					 						{
					 							fDataValuePtr = CP_new CP_DataValueTemp<T>( inValue );
					 						}

	//! Destructor.
										/*!
										*	Destructor.
										*/
	virtual								~CP_DataValueHolder()										{ delete fDataValuePtr; }
	//@}
			
	//! @name Getters
	//@{
										/*!
										*	Returns the CP_DataValue pointer to the data value we are holding.
										*
										*	@return	CP_DataValue	Returns a pointer to the data value.
										*/
			CP_DataValue				*GetDataValuePtr() const									{ return( fDataValuePtr ); }

										/*!
										*	Returns a bool value if the data value is of type T.
										*
										*	@return	bool	Returns true if the value if of type T.
										*/
			 							template <typename T>
			bool 						Get( T& inDataValue ) const
											{
												const CP_DataValueTemp<T>		*dataValuePtr = dynamic_cast<CP_DataValueTemp<T> *>(fDataValuePtr);
												
												if( dataValuePtr )
												{
													inDataValue = dataValuePtr->GetDataValue();

													return( true );
												}
												
												return( false );
											}
		
										/*!
										*	Returns the data value as a type T. Throws std::bad_cast if the cast was unsuccessful.
										*
										*	@return	T		Returns data value as a type T.
										*/
			 							template <typename T>
			const T& 					Get() const
											{
												const CP_DataValueTemp<T> *dataValuePtr = dynamic_cast<CP_DataValueTemp<T>*>(fDataValuePtr);
												
												if( dataValuePtr == NULL )
													CP_ThrowBadCast( CP_String("Get data value bad cast.") );
													
												return( dataValuePtr->GetDataValue() );
											}

										/*!
										*	Returns the type of our data value as a std::type_info.
										*
										*	@return	std::type_info		Returns the type of our data value.
										*/
			const std::type_info& 		type() const
											{
												ASSERT( fDataValuePtr != NULL );
																						
												return( fDataValuePtr->type() );
											}
	//@}
			
	//! @name operators
	//@{
										/*!
										*	Assignment operator. Copies the value from the passed in data value holder to
										*	this object. Template version.
										*
										*	@return	CP_DataValueHolder		Returns a reference to ourselves.
										*/
			CP_DataValueHolder& 		operator=( const CP_DataValueHolder& inHolder )
										{
											//fDataValuePtr = inHolder.fDataValuePtr;
											
											//return( *this );
											CP_DataValueHolder		temp( inHolder );
											
											return( swap( temp ) );
										}

	#if ! TARGET_CC_MSVC

										/*!
										*	Assignment operator. Copies the value from the passed in data value holder to
										*	this object. Template version.
										*
										*	@return	CP_DataValueHolder		Returns a reference to ourselves.
										*/
			template <typename T>
			CP_DataValueHolder& 		operator=( const T& inHolder )
										{
											return( swap( CP_DataValueHolder( inHolder ) ) );
										}
	//@}
	#endif

	//! @name Misc
	//@{
										/*!
										*	Swaps our data value with that of the passed in data value holder.
										*
										*	@return	CP_DataValueHolder&		Returns a reference to ourselves.
										*/
			CP_DataValueHolder& 		swap( CP_DataValueHolder& inHolder )
										{
											std::swap( fDataValuePtr, inHolder.fDataValuePtr );
											
											return( *this );
										}
	//@}

private:

	// class data
private:
	CP_DataValue						*fDataValuePtr;		/*! Pointer to a CP_DataValue object that we are holding. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_DataValueHolder
