//=========================================================================
//
//	FILE:				CP_Debugging.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Various macros, routines and other defines to help
//						in debugging CPLAT apps.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 196 $
//						Latest Checkout:  $Date: 2005-05-31 16:14:17 -0700 (Tue, 31 May 2005) $
//
//=========================================================================
#ifndef _H_CP_Debugging
#define _H_CP_Debugging
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_LogFile.h"
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

class CP_String;
class CP_LogFile;

/*!
*	@namespace	CP_Debug
*	@brief		A grouping of macros, function and a class to handle debugging.
*	
*	This group of functions, macros and a class are using to handle preseting
*	debugging information to the user in a CPLAT application. 
*/
namespace CP_Debugging
{

/*! Enumeration to defines how notifications will be handled. */
typedef enum
{
	CP_E_DebugNotification_Nothing				= 0,	/*!< Nothing happens on notifications. */
	CP_E_DebugNotification_Alert				= 1,	/*!< An alert is show to user. */
	CP_E_DebugNotification_Debugger				= 2,	/*!< The notifiction goes to the debugger. */
	CP_E_DebugNotification_Console				= 3,	/*!< The notification is written to the console. */
	CP_E_DebugNotification_LogFile				= 4		/*!< The notification is written to a log file. */
} CP_E_DebugNotification;

class CP_EXPORT CP_DebugNotification
{
public:
	//! @name Debugger
	//@{
	static void							CP_Debugger();
	//@}
	
	//! @name Error Notifications
	//@{
										/*!
										*	Returns how exception notification is handled.
										*	
										*	@exception	none
										*	
										*	@return		CP_E_DebugNotification		Flag indicating how exception notification is handled.
										*/
	static CP_E_DebugNotification		GetExceptionNotification()												{ return( gExceptionNotification ); }

	static void							SetExceptionNotification( CP_E_DebugNotification inNotification );

										/*!
										*	Returns a flag to indicate if exceptions should be reported
										*	when they are thrown.
										*	
										*	@exception	none
										*	
										*	@return		CP_E_DebugNotification			Flag indicating how exception notification is handled.
										*/
	static bool							GetReportExceptionOnThrow()												{ return( gReportExceptionOnThrow ); }

										/*!
										*	Sets the flag to indicate is exceptions should be reported
										*	when they are thrown.
										*	
										*	@param		inReport		If true, exceptions are reported when they are thrown.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	static void							SetReportExceptionOnThrow( bool inReport )								{ gReportExceptionOnThrow = inReport; }

										/*!
										*	Returns how error notification is handled.
										*	
										*	@exception	none
										*	
										*	@return		CP_E_DebugNotification			Flag indicating how error notification is handled.
										*/
	static CP_E_DebugNotification		GetErrorNotification()													{ return( gErrorNotification ); }

										/*!
										*	Sets the flag to indicate how error notification is handled.
										*	
										*	@param		inNotification		Indicates how the error notification is handled.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	static void							SetErrorNotification( CP_E_DebugNotification inNotification )			{ gErrorNotification = inNotification; }

										/*!
										*	Returns how warning notification is handled.
										*	
										*	@exception	none
										*	
										*	@return		CP_E_DebugNotification			Flag indicating how warning notification is handled.
										*/
	static CP_E_DebugNotification		GetWarningNotification()												{ return( gWarningNotification ); }

										/*!
										*	Sets the flag to indicate how warning notification is handled.
										*	
										*	@param		inNotification		Indicates how the warning notification is handled.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	static void							SetWarningNotification( CP_E_DebugNotification inNotification )			{ gWarningNotification = inNotification; }
										/*!
										*	Returns how failed assertion notification is handled.
										*	
										*	@exception	none
										*	
										*	@return		CP_E_DebugNotification			Flag indicating how failed assertion notification is handled.
										*/
	static CP_E_DebugNotification		GetAssertionNotification()												{ return( gAssertionNotification ); }

										/*!
										*	Sets the flag to indicate how failed assertion notification is handled.
										*	
										*	@param		inNotification		Indicates how the failed assertion notification is handled.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	static void							SetAssertionNotification( CP_E_DebugNotification inNotification )			{ gAssertionNotification = inNotification; }
	//@}

	//! @name LogFile
	//@{
	static CP_LogFile					*GetDebugLogFile();

										/*!
										*	Sets the file spec to be used for the debugging log file.
										*
										*	@param		inSpec				The file system spec of the logging file.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	static void							SetDebugLogFileSpec( const CP_FileSystemSpec& inSpec )					{ gDebugLogFileSpec = inSpec; gUserSuppliedFileSpec = true; }
	//@}

	// class data
private:
	static CP_E_DebugNotification		gExceptionNotification;			/*! Indicates how exceptions notifications take place. */
	static CP_E_DebugNotification		gErrorNotification;				/*! Indicates how error notifications take place. */
	static CP_E_DebugNotification		gWarningNotification;			/*! Indicates how warning notifications take place. */
	static CP_E_DebugNotification		gAssertionNotification;			/*! Indicates how notifications about failed assertions take place. */

	static bool							gReportExceptionOnThrow;		/*! Flag to indicate if we report exceptions when they are thrown. */
	
	static CP_LogFile					*gDebugLogFile;					/*! Log file to report debugging to. */
	static CP_FileSystemSpec			gDebugLogFileSpec;				/*! CP_FileSystemSpec for debugging log file. */
	static bool							gUserSuppliedFileSpec;			/*! Has user supplied a file spec for the log file. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_GNUC
	#define __CP_DEBUGGING_FUNCTIONNAME__ __PRETTY_FUNCTION__
#elif TARGET_CC_MSVC
	#define __CP_DEBUGGING_FUNCTIONNAME__ ""
#else
	#define __CP_DEBUGGING_FUNCTIONNAME__ __FUNCTION__
#endif
/*!
*	The macros and functions defined in the CP_Debugging file are designed to
*	aid in the debugging process of a CPLAT application. These define macros
*	for handling assertions.
*/

#define CP_ReportException( description, inException )	CPLAT::CP_ErrorManager::Instance()->ReportException( description, inException, __CP_DEBUGGING_FUNCTIONNAME__, __LINE__ , __FILE__);
#define CP_ReportError( description, message )			CPLAT::CP_ErrorManager::Instance()->ReportError( description, message, __CP_DEBUGGING_FUNCTIONNAME__, __LINE__ , __FILE__ );
#define CP_ReportWarning( description, message )		CPLAT::CP_ErrorManager::Instance()->ReportWarning( description, message, __CP_DEBUGGING_FUNCTIONNAME__, __LINE__ , __FILE__ );

//========================================================================
// Assertion macros
//========================================================================
#if CP_Debug

	//! @result bool	whether to break into the debugger or not
	bool CP_ReportAssertFailed( const CP_String& inMessage, const CP_String& inDescription, const char *inFunction, const char *inFile, int inLine  );

	#if TARGET_OS_WIN32
		#define CP_DebugBreak() DebugBreak()
	#elif TARGET_OS_MAC
		#define CP_DebugBreak() Debugger()
	#endif
		// Assertions

	#if TARGET_OS_LINUX
		#define ASSERT(test)								{}
		#define ASSERT_MSG(test, msg)						{}

		#define CP_CHECK_PTR(p)								{}
		#define CP_CHECK_HANDLE(h)							{}

		inline void DebugTrace( ... ) {}
	#else
		#define ASSERT( test )          	(void) ((test) || !CPLAT::CP_Debugging::CP_ReportAssertFailed(#test, "", __CP_DEBUGGING_FUNCTIONNAME__, __FILE__, __LINE__) || (CP_DebugBreak(), 0))
		#define ASSERT_MSG( test, msg )     (void) ((test) || !CPLAT::CP_Debugging::CP_ReportAssertFailed(#test, msg,  __CP_DEBUGGING_FUNCTIONNAME__, __FILE__, __LINE__) || (CP_DebugBreak(), 0))

		#define CP_CHECK_PTR(p)         	(void) ((p != NULL) || !CPLAT::CP_Debugging::CP_ReportAssertFailed("Bad pointer.", "", __CP_DEBUGGING_FUNCTIONNAME__, __FILE__, __LINE__) || (CP_DebugBreak(), 0))
		#define CP_CHECK_HANDLE(h)          (void) ((h != NULL && *h != NULL) || !CPLAT::CP_Debugging::CP_ReportAssertFailed("Bad handle.", "", __CP_DEBUGGING_FUNCTIONNAME__, __FILE__, __LINE__) || (CP_DebugBreak(), 0))
			
		extern CP_LogFile& gTraceLog;
		
		void DebugTrace( char *inFormatStr, ... );
	#endif	
#else	// CP_Debug
		// Assertions
	#define ASSERT(test)								{}
	#define ASSERT_MSG(test, msg)						{}

	#define CP_CHECK_PTR(p)								{}
	#define CP_CHECK_HANDLE(h)							{}

	inline void DebugTrace( ... ) {}
	
#endif	// ! CP_Debug

}	// namespace CP_Debugging

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Debugging
