//=========================================================================
//
//	FILE:				CP_ImageList.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			An image list maintains a series of bitmap images for use
//						in various display classes.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				PLATFORM SPECIFICS:
//						MacOS - Since the MacOS doesn't have a concept of an
//							image list, we simple store the images into an
//							offscreen GWorld that is imageHeight high by number
//							of images times their length SInt32.
//						Windows - We provide a wrapper for the ImageList structures
//							and functions.
//
//	VERSION:			Current Revision: $Rev: 220 $
//						Latest Checkout:  $Date: 2005-06-26 16:44:04 -0700 (Sun, 26 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_ImageList
#define _H_CP_ImageList
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_Enums.h"
#include "CP_OSIcon.h"
#include "CP_M_ReferenceCounted.h"
#include "CP_OffscreenCanvas.h"
#include "CP_Point.h"
#include "CP_Rect.h"
#include "CP_RGBColor.h"
#include "CP_Types.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_LINUX
	typedef std::vector<char **>									CP_Image_Array;
	typedef std::vector<char **>::iterator							CP_Image_Iterator;
	typedef std::vector<char **>::reverse_iterator					CP_Image_ReverseIterator;
#endif

class CP_EXPORT CP_ImageList : public CP_M_RefCounted
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ImageList();
										CP_ImageList(
												SInt16 inNumImages,
												SInt16 inImageWidth = 0,
												SInt16 inImageHeight = 0,
												bool inIsMasked = true );
	//! Destructor.
	virtual								~CP_ImageList();
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the platforms OS image list.
										*	
										*	@exception	none
										*	
										*	@return		OSImageList		The platforms OS image list.
										*/
			OSImageList					GetPlatformImageList()				{ return( fImages ); }

										/*!
										*	Returns the width of an image.
										*	
										*	@exception	none
										*	
										*	@return		SInt16			Width of a single image.
										*/
			SInt16						GetWidth()							{ return( fImageWidth ); }

										/*!
										*	Returns the height of an image.
										*	
										*	@exception	none
										*	
										*	@return		SInt16			Height of a single image.
										*/
			SInt16						GetHeight()							{ return( fImageHeight ); }

										/*!
										*	Return the slop value used to determine if a color is close enough. Used in calculating masks.
										*	
										*	@exception	none
										*	
										*	@return		UInt16			Value used to determine how close a color
										*								has to be to be considered equal in generating
										*								mask data.
										*/
			UInt16						GetSlopValue()						{ return( fSlopValue ); }		
			
			SInt16						GetNumberImages();

										/*!
										*	Return the transfer mode used to copy images back to the destination canvas.
										*	
										*	@exception	none
										*	
										*	@return		SInt32			The transfer mode.
										*/
			SInt32						GetTransferMode()					{ return( fTransferMode ); }
	//@}

	//! @name Images
	//@{
			void						Add( SInt16 inImageID );
			void						AddMasked(
												SInt16 inImageID,
												CP_RGBColor& inMaskColor,
												bool inUseZeroPixelColor = false );

			void						AddIconImage( SInt16 inImageID );
			void						AddIconImage( CP_OSIcon *inImage );

			void						Remove( SInt16 inImageIndex );
			void						RemoveAll();
	//@}
			
	//! @name Drawing
	//@{
			void						CopyImageTo(
											CP_Canvas& inCanvas,
											SInt16 inImageIndex,
											const CP_Point& inDestLocation,
											bool inShowSelected = false,
											bool isActive = true );
	//@}

	//! @name Setters
	//@{
			void						SetImageSize(
												SInt16 inHorSize,
												SInt16 inVerSize );

										/*!
										*	Sets the slop value which is used in determining how close a color
										*	must be to be considered a match when generating the masks for an
										*	image.
										*
										*	@param		inSlopValue		The value to add to each side of a color
										*								when comparing.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetSlopValue( UInt16 inSlopValue )	{ fSlopValue = inSlopValue; }		

										/*!
										*	Return the transfer mode used to copy images back to the destination canvas.
										*
										*	@param		inMode			The mode used to copy image.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetTransferMode( SInt16 inMode )	{ fTransferMode = inMode; }
	//@}

private:
	//! @name Initialization
	//@{
			void						Init(
												SInt16 inNumImages = 0,
												SInt16 inImageWidth = 0,
												SInt16 inImageHeight = 0,
												bool inIsMasked = true );
	//@}
			
	// class data
protected:
								
	OSImageList							fImages;			/*! Contains the images arranged in a bitmap
																imageWidth * numberOfImages horizontally, and
																imageHeight vertically. */
	OSImageList							fMasks;				/*! Contains the masks for the above images,
																stored the same way. */
	SInt16								fImageWidth;		/*! Width in pixels of a single image. */
	SInt16								fImageHeight;		/*! Height in pixels of a single image. */
	SInt16								fNumberImages;		/*! Number of images stored in the list. */
	SInt16								fMaxNumberImages;	/*! The maximum number of images we can store. */
	SInt16								fImageGrowAmt;		/*! Number of images to increase the bitmap,
																when we resize to accommodate more images. */

	bool								fIsMasked;			/*! Flag to indicate if the images are masked. */
	UInt16 								fSlopValue;			/*! Value used to calculate if colors are close enough to use as a mask. */
	SInt32								fTransferMode;		/*! Transfer mode used to copy the images to the destination canvas. */
	
	#if TARGET_OS_MAC
		GWorldPtr						fSavedPort;			/*! MacOS saved GWorld when drawing into our image list. */
		GDHandle						fSavedDevice;		/*! MacOS saved GDHandle when drawing into our image list. */
	#endif

	#if TARGET_OS_LINUX
		CP_Image_Array					*fLinuxImages;
	#endif

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Images
	//@{
			void						AddImage(
												CP_OffscreenCanvas *inCanvas,
												const CP_Rect& inSourceArea );

			void						AddMasked_Mac(
												PicHandle inPicture,
												CP_RGBColor& inMaskColor,
												bool inUseZeroPixelColor );

			void						LoadImages( SInt16 inResourceID );
	//@}

	//! @name Storage
	//@{
	virtual	void						GrowBitmapStorage();
	//@}

private:
	//! @name Misc
	//@{
			void						AllocGWorlds(
												SInt16 inNumImages,
												SInt16 inImageWidth,
												SInt16 inImageHeight );
	//@}

	//! @name Images
	//@{
			void						AddPictImage_Mac( SInt16 inImageID );
			void						AddMasked_Mac(
												SInt16 inImageID,
												CP_RGBColor& inMaskColor,
												bool inUseZeroPixelColor = false );

			void						AddIconImage_Mac( SInt16 inImageID );
			void						AddIconImage_Mac( CP_OSIcon *inImage );

			void						CopyIconData_Mac( OSColorIcon inIconData );
			void						CopyIconData_Mac_IconSuite( IconSuiteRef inIconData );
			void						CopyIconData_Mac_IconReference( IconRef inIconData );

			void						CopyImageTo_Mac(
												CP_Canvas& inCanvas,
												SInt16 inImageIndex,
												const CP_Point& inDestLocation,
												bool inShowSelected = false,
												bool isActive = true );

			void						CreateRGBMask_Mac(
												CP_RGBColor& inMaskColor,
												const CP_Rect& inImageArea,
												bool inUseZeroPixelColor = false );
	//@}
#endif

#if TARGET_OS_WIN32
public:
	//! @name Images
	//@{
			void						LoadImages( SInt16 inResourceID );
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_ImageList

