//=========================================================================
//
//	FILE:				CP_UndoPolicy_Multi.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Undo policy class for objects that supports undo / redo.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 194 $
//						Latest Checkout:  $Date: 2005-05-25 22:20:49 -0700 (Wed, 25 May 2005) $
//
//=========================================================================
#ifndef _H_CP_UndoPolicy_Multi
#define _H_CP_UndoPolicy_Multi
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_M_CommandHandler.h"
#include "CP_NoCopy.h"
#include "CP_UndoAction.h"

	// Standard Library
#include <list>

CPLAT_Begin_Namespace_CPLAT

	// typedefs
#if ! TARGET_OS_LINUX
	typedef std::list<CP_UndoAction *, std::allocator<CP_UndoAction *> >				CP_UndoAction_List;
	typedef std::list<CP_UndoAction *, std::allocator<CP_UndoAction *> >::iterator		CP_UndoAction_Iterator;
#else
	typedef std::list<CP_UndoAction *>													CP_UndoAction_List;
	typedef std::list<CP_UndoAction *>::iterator										CP_UndoAction_Iterator;
#endif

/*!
*	@class		CP_UndoPolicy_Multi
*	@brief		Undo policy class for objects that supports undo / redo.
*/
class CP_EXPORT CP_UndoPolicy_Multi : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_UndoPolicy_Multi();
	//! Destructor.
	virtual								~CP_UndoPolicy_Multi() throw();
	//@}
			
	//! @name Commands
	//@{
			void						HandlePostCommand( CP_UndoAction *inCommand );
			void						HandleUndoCommand( OSStatus& inOutStatus );
			void						HandleRedoCommand( OSStatus& inOutStatus );
	//@}

	//! @name Getters
	//@{
			CP_String					GetUndoCommandText();
			CP_String					GetRedoCommandText();

										/*!
										*	Return the undo menu item text.
										*	
										*	@exception	none
										*	
										*	@return		CP_String		The text to be used.
										*/
			CP_String					GetUndoText()											{ return( fUndoText ); }

										/*!
										*	Return the redo menu item text.
										*	
										*	@exception	none
										*	
										*	@return		CP_String		The text to be used.
										*/
			CP_String					GetRedoText()											{ return( fRedoText ); }

										/*!
										*	Return the can't undo menu item text.
										*	
										*	@exception	none
										*	
										*	@return		CP_String		The text to be used.
										*/
			CP_String					GetCantUndoCommandText()								{ return( fCantUndoText ); }

										/*!
										*	Return the can't redo menu item text.
										*	
										*	@exception	none
										*	
										*	@return		CP_String		The text to be used.
										*/
			CP_String					GetCantRedoCommandText()								{ return( fCantRedoText ); }
	//@}

	//! @name List Handling
	//@{
			void						ClearHistory();
			void						ClearRedoHistory();
			void						ClearUndoHistory();
			void						ClearCommand(
												CP_UndoAction *inCommand,
												bool inUndoHistory = true,
												bool inRedoHistory = true );
	//@}

	//! @name Menu Status
	//@{
			void						UpdateUndoMenuStatus(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommand );
			void						UpdateRedoMenuStatus(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommand );
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the maximum number of commands to keep in the list
										*	of undo / redo commands.
										*
										*	@param		inMaxNumber		The maximum number of commands.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetMaxNumberCommands( SInt16 inMaxNumber )				{ fMaxNumberCommands = inMaxNumber; }

										/*!
										*	Sets the text to be used for the undo menu item.
										*
										*	@param		inUndoText		The text to be used for undo commands.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual	void						SetUndoCommandText( CP_String& inUndoText )			{ fUndoText = inUndoText; }

										/*!
										*	Sets the text to be used for the redo menu item.
										*
										*	@param		inUndoText		The text to be used for redo commands.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual	void						SetRedoCommandText( CP_String& inRedoText )			{ fRedoText = inRedoText; }

										/*!
										*	Sets the text to be used for the can't undo menu item.
										*
										*	@param		inCantUndoText	The text to be used for cant undo commands.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual	void						SetCantUndoCommandText( CP_String& inCantUndoText )	{ fCantUndoText = inCantUndoText; }

										/*!
										*	Sets the text to be used for the can't redo menu item.
										*
										*	@param		inCantRedoText	The text to be used for cant redo commands.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual	void						SetCantRedoCommandText( CP_String& inCantRedoText )	{ fCantRedoText = inCantRedoText; }
	//@}

	//! @name Testers
	//@{
			bool						HasUndoCommand( CP_UndoAction *inCommand );
			bool						HasRedoCommand( CP_UndoAction *inCommand );
	//@}

	//! @name Undo / Redo
	//@{
	virtual	bool						CanUndo();
	virtual	bool						CanRedo();

	virtual	void						DoUndo();
	virtual	void						DoRedo();
	//@}

protected:
	//! @name Lists
	//@{
										/*!
										*	Returns the undo actions list, which contains a list of the actions
										*	that can currently be undone.
										*	
										*	@exception	none
										*	
										*	@return		CP_UndoAction_List		List of actions that can be undone.
										*/
			CP_UndoAction_List			*GetUndoCommandsList()									{ return( fUndoCommands ); }

										/*!
										*	Returns the redo actions list, which contains a list of the actions
										*	that can currently be redone.
										*	
										*	@exception	none
										*	
										*	@return		CP_UndoAction_List		List of actions that can be redone.
										*/
			CP_UndoAction_List			*GetRedoCommandsList()									{ return( fRedoCommands ); }
	//@}

	// class data
protected:
	SInt16								fMaxNumberCommands;		/*! Maximum number of commands we maintain. */
	CP_UndoAction_List					*fUndoCommands;			/*! List of undoable commands. */
	CP_UndoAction_List					*fRedoCommands;			/*! List of redoable commands. */

	CP_String							fUndoText;				/*! Undo menu item text. */
	CP_String							fRedoText;				/*! Redo menu item text. */
	CP_String							fCantUndoText;			/*! Can't undo menu item text. */
	CP_String							fCantRedoText;			/*! Can't redo menu item text. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_UndoPolicy_Multi

