//=========================================================================
//
//	FILE:				CP_FileSystemDialogsBase.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Base class for a group of dialogs for dealing with the platforms
//						file system dialogs / navigation services.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 162 $
//						Latest Checkout:  $Date: 2005-04-28 15:05:07 -0700 (Thu, 28 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_FileSystemDialogs_Base
#define _H_CP_FileSystemDialogs_Base
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Enums.h"
#include "CP_FileSystemSpec.h"
#include "CP_SignalSlot.h"
#include "CP_Window.h"

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_FileSystemDialogs_Base;

typedef CP_SigSlot::CP_Signal3<CP_FileSystemDialogs_Base *, const CP_FileSystemSpec&, bool>		CP_FileSelectionCallback;
typedef CP_SigSlot::CP_Signal2<CP_FileSystemDialogs_Base *, CP_E_NavDialogResult>				CP_SaveChangesCallback;

/*!
*	@class		CP_FileSystemDialogs_Base
*	@brief		Base class for a group of dialogs for dealing with the platforms file system dialogs / navigation services.
*/
class CP_EXPORT CP_FileSystemDialogs_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FileSystemDialogs_Base();
	//! Destructor.
	virtual								~CP_FileSystemDialogs_Base() throw();
	//@}
			
	//! @name Events
	//@{
			void						Run();
	//@}
			
	//! @name Setters
	//@{
			void						SetModality(
												WindowModality inModality,
												CP_Window *inParent = NULL );

										/*!
										*	Sets the callback when the user makes a file selection.
										*	
										*	@param		inCallback				The function called upon the selection.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetFileSelectionCallback( CP_FileSelectionCallback *inCallback )	{ fSelectionCallback = inCallback; }

										/*!
										*	Sets the callback when the user chooses to save changes.
										*	
										*	@param		inCallback				The function called when user
										*										chooses to save changes.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetSaveChangesCallback( CP_SaveChangesCallback *inCallback )		{ fSaveChangesCallback = inCallback; }

										/*!
										*	Sets the callback when the user chooses to revert to last saved file.
										*	
										*	@param		inCallback				The function called when user
										*										chooses to revert.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetRevertCallback( CP_SaveChangesCallback *inCallback )				{ fRevertCallback = inCallback; }

										/*!
										*	Sets the callback when the user chooses not to save changes.
										*	
										*	@param		inCallback				The function called when user
										*										chooses not to save changes.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetDontSaveChangesCallback( CP_SaveChangesCallback *inCallback )	{ fDontSaveChangesCallback = inCallback; }

										/*!
										*	Sets the callback when the user cancels  Navigation Services dialog.
										*	
										*	@param		inCallback				The function called when user
										*										cancels Navigation Services dialog.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetCancelCallback( CP_SaveChangesCallback *inCallback )				{ fCancelCallback = inCallback; }
			
			void						SetAppName( const CP_String& inAppName );
			void						SetFileName( const CP_String& inFileName );
	//@}

protected:

	// class data
protected:
	CP_FileSelectionCallback			*fSelectionCallback;			/*! Signal called when the user selects a file. */
	CP_SaveChangesCallback				*fSaveChangesCallback;			/*! Signal called when the user responds to the AskSaveChanges dialog. */
	CP_SaveChangesCallback				*fRevertCallback;				/*! Signal called when the user responds to the AskRevert dialog. */
	CP_SaveChangesCallback				*fDontSaveChangesCallback;		/*! Signal called when the user responds to the AskSaveChanges/AskRevert dialog. */
	CP_SaveChangesCallback				*fCancelCallback;				/*! Signal called when the user responds to the AskSaveChanges/AskRevert dialog. */

	#if TARGET_OS_MAC
		NavDialogRef					fNavDialog;						/*! MacOS NavDialogRef for the current Navigation Services dialog. */
		bool							fDialogIsRunning;				/*! Flag to indicate the dialog is currently running. */

		NavDialogCreationOptions		fNavDialogOptions;				/*! Options used to control appearance and behavior of Navigation Services dialog. */
	#else
		CP_String						fFileName;						/*! Filename of file selected in dialogs. */
	#endif
			
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Callbacks
	//@{
	static pascal void					NavEventProc(
												NavEventCallbackMessage inCallBackSelector, 
												NavCBRecPtr inCallBackParms, 
												void *inCallBackUD ) throw();
	//@}
	
	//! @name Getters
	//@{
			NavUserAction				GetUserAction() const;

										/*!
										*	Returns the MacOS NavDialogRef for the current Navigation Services dialog.
										*	
										*	@exception	none
										*
										*	@return		NavDialogRef			The MacOS Navigation Services dialog ref.
										*/
			NavDialogRef				GetNavDialog()											{ return( fNavDialog ); }

										/*!
										*	Returns the MacOS NavDialogOptionFlags for the current Navigation Services dialog.
										*	These are the options that control the appearance and behavior of the dialog.
										*	
										*	@exception	none
										*
										*	@return		NavDialogOptionFlags	The MacOS Navigation Services NavDialogOptionFlags.
										*/
			NavDialogOptionFlags		GetOptionFlags()										{ return( fNavDialogOptions.optionFlags ); }
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the MacOS NavDialogOptionFlags for the current Navigation Services dialog.
										*	These are the options that control the appearance and behavior of the dialog.
										*	
										*	@param		inFlags					The flags to be used.
										*	
										*	@exception	none
										*
										*	@return		NavDialogOptionFlags	The MacOS Navigation Services NavDialogOptionFlags.
										*/
			void						SetOptionFlags( NavDialogOptionFlags inFlags )			{ fNavDialogOptions.optionFlags |= inFlags; }
	//@}
		
protected:
	//! @name Dialog handlers
	//@{
										/*!
										*	Virtual function that each class implements to create the specific dialog to
										*	implement the Navigation Services handling of the request.
										*	
										*	@param		inOptions				NavDialogCreationOptions used to create the dialog.
										*	@param		inEventUPP				The event handling UPP.
										*	@param		inUserData				Used data passed between this class and the Navigation Services
										*										Dialog.
										*	
										*	@exception	none
										*
										*	@return		NavDialogOptionFlags	The MacOS Navigation Services NavDialogOptionFlags.
										*/
	virtual	NavDialogRef				CreateDialog(
												NavDialogCreationOptions& inOptions, 
												NavEventUPP inEventUPP,
												void *inUserData ) = 0;

	virtual	void						HandleEvent(
												NavEventCallbackMessage inSelector, 
												NavCBRec& ioParams );

	virtual void						HandleNavUserAction( NavUserAction inUserAction );

	virtual	void						RunPlatformDialog();

			CP_E_NavDialogResult		GetNavDialogResult( NavUserAction inUserAction );

	//@}

	//! @name Callbacks
	//@{
	virtual	void						HandleNavStart( NavCBRec& inOutParams );
	virtual	void						HandleNavTerminate();
	//@}

private:
	//! @name Cleanup
	//@{
			void						PlatformCleanUp();
	//@}
#endif

#if TARGET_OS_WIN32
protected:
	//! @name Dialog handlers
	//@{
	virtual	void						RunPlatformDialog();
	//@}

private:
	//! @name Cleanup
	//@{
			void						PlatformCleanUp();
	//@}
#endif

#if TARGET_OS_LINUX
protected:
	//! @name Dialog handlers
	//@{
	virtual	void						RunPlatformDialog();
	//@}

private:
	//! @name Cleanup
	//@{
			void						PlatformCleanUp();
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_FileSystemDialogs_Base

