//=========================================================================
//
//	FILE:				CP_EventHandler_Imp_CE.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation class for the CP_EventHandler when
//						using MacOS Carbon Events.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_EventHandler_Imp_CE
#define _H_CP_EventHandler_Imp_CE
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC && TARGET_API_MAC_CARBON

	// CPLAT_Framework
#include "CP_Event.h"
#include "CP_New.h"
#include "CP_NoCopy.h"
#include "CP_OSStatus.h"
#include "CP_SignalSlot_Defines.h"
#include "CP_Types.h"

	// Standard Library
#include <map>
#if TARGET_CC_METROWERKS
	#include <hash_map>
#endif

CPLAT_Begin_Namespace_CPLAT

	// Signals and slots

/*!
*	@struct		Less
*	@brief		Map functor for comparing our EventTypeSpecs
*
*	@ingroup	Events
*/
struct Less : public std::binary_function<EventTypeSpec, EventTypeSpec, bool>
{
								/*!
								*	operator () called when we need to compare the passed in EventTypeSpecs
								*	
								*	@exception	none
								*	
								*	@return		bool const		Returns true if lhs spec is less than rhs spec.
								*/
	bool						operator()(const EventTypeSpec& lhs, const EventTypeSpec& rhs) const
									{
										return( (lhs.eventClass < rhs.eventClass) || 
											((lhs.eventClass == rhs.eventClass) && (lhs.eventKind < rhs.eventKind)));
									}
};

/*!
*	@struct		Hash
*	@brief		Hash Map functor for generating the hash key our EventTypeSpecs
*
*	@ingroup	Events
*/
struct Hash : public std::unary_function<EventTypeSpec, size_t>
{
								/*!
								*	operator () called when we need to generate the hash for the passed in EventTypeSpecs
								*	
								*	@exception	none
								*	
								*	@return		size_t const		Returns the hash key.
								*/
	size_t						operator()(const EventTypeSpec& key) const
								{
									return (key.eventClass | key.eventKind);
								}
};

/*!
*	@struct		Equal
*	@brief		Hash Map functor for comparing our EventTypeSpecs
*
*	@ingroup	Events
*/
struct Equal : public std::binary_function<EventTypeSpec, EventTypeSpec, bool>
{
								/*!
								*	operator () called when we need to compare the passed in EventTypeSpecs
								*	for equality.
								*	
								*	@exception	none
								*	
								*	@return		bool const		Returns true if lhs spec is equal to in rhs spec.
								*/
	bool						operator()(const EventTypeSpec& lhs, const EventTypeSpec& rhs) const
								{
									return( (lhs.eventClass == rhs.eventClass) && (lhs.eventKind == rhs.eventKind) );
								}
};

/*! CP_EventHandlerMap
*
*	Holds a mapping between an EventTypeSpec and the corresponding CP_EventSignal to handle the event.
*/
#if TARGET_CC_METROWERKS
		// prefer hash_map if available for a little more speed
	typedef Metrowerks::hash_map<EventTypeSpec, CP_EventSignal *, Hash, Equal >				CP_EventHandler_Map;
	typedef Metrowerks::hash_map<EventTypeSpec, CP_EventSignal *, Hash, Equal >::iterator	CP_EventHandler_Iterator;
#else
	typedef std::map<EventTypeSpec, CP_EventSignal *, Less >								CP_EventHandler_Map;
	typedef std::map<EventTypeSpec, CP_EventSignal *, Less >::iterator						CP_EventHandler_Iterator;
#endif

typedef std::pair<const EventTypeSpec, CP_EventSignal *>									CP_EventHandler_Entry;

/*!
*	@class		CP_EventHandler_Imp
*	@brief		Implementation class for the CP_EventHandler when using MacOS Carbon Events.
*
*	@ingroup	Events
*/
class CP_EXPORT CP_EventHandler_Imp : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_EventHandler_Imp();
								CP_EventHandler_Imp( EventTargetRef inTarget );
	//! Destructor.
	virtual						~CP_EventHandler_Imp();
	//@}
			
	//! @name Events
	//@{
			void				Add(
										UInt32 inClass, 
										UInt32 inKind, 
										CP_EventSignal *inEventSignal,
										OSWindow inWindow = NULL );

			void				Remove(
										UInt32 inEventClass,
										UInt32 inEventKind );

			bool				HasHandler(
										UInt32 inEventClass,
										UInt32 inEventKind );

			CP_EventSignal		*FindHandler(
										UInt32 inEventClass,
										UInt32 inEventKind );
	//@}
	
	//! @name Getters
	//@{
			EventTargetRef		GetEventTargetRef()											{ return( fTarget ); }
	//@}

	//! @name Setters
	//@{
			void				SetEventTargetRef( EventTargetRef inTarget )				{ fTarget = inTarget; }
	//@}

protected:

private:
	//! @name Callback
	//@{
	static pascal ::OSStatus 	EventHandlerProc(
										EventHandlerCallRef inHandlerCallRef, 
										EventRef inEvent, 
										void *inUserData ) throw();
	//@}

	// class data
protected:
	CP_EventHandler_Map			fEventHandlers;		/*! Map that contains all of the registered handlers. */
	EventTargetRef				fTarget;			/*! Carbon Event target ref. */
	EventHandlerRef				fEventHandlerRef;	/*! Carbon Event handler ref. */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC && TARGET_API_MAC_CARBON

#endif	// _H_CP_EventHandler_Imp_CE

