//=========================================================================
//
//	FILE:				CP_FileSystemSpec_Imp_Mac.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			MacOS implementation of the CP_FileSystemSpec_Imp class.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				Since an FSRef is unable to specify a non existant file
//						we use the data member fFileExists to indicate if the
//						fFileRef is pointing to a valid file if true, otherwise
//						the fFileRef is the parent directory for our file specified
//						by fFileName.
//
//	VERSION:			Current Revision: $Rev: 205 $
//						Latest Checkout:  $Date: 2005-06-07 20:17:23 -0700 (Tue, 07 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_FileSystemSpec_Imp_Mac
#define _H_CP_FileSystemSpec_Imp_Mac
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_String.h"

	// Standard Library
#include <stdio.h>
#include <string>

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_CFURL;

/*!
*	@class	CP_FileSystemSpec_Imp
*	@brief	MacOS implementation of the CP_FileSystemSpec_Imp class.
*	
*	This class implements the CP_FileSystemSpec for the MacOS platform, and
*	has methods you can call directly, or are called by the framework for
*	MacOS specific features.
*/
class CP_EXPORT CP_FileSystemSpec_Imp
{
public:
	/*! @brief Smart FSRef result class.
	    
		This convenience class is returned wherever an FSRef would be returned. The class is essentially the
	    same as an FSRef, but it can pose as an FSRef pointer, so the result of calls returning 
		an FSRef can be passed directly into calls expecting a const FSRef* (like e.g. all of the 
		FileManager calls).
	*/
	struct FSRefResult : public FSRef
	{
	public:	
		//! Default constructor.
		FSRefResult() {}
		//! Copy constructor.
		explicit FSRefResult(const FSRef& fsRef) : FSRef(fsRef) {}
		//! Assignment
		FSRefResult& operator = (const FSRef& fsRef) { CP_STD::memcpy(this, &fsRef, sizeof(FSRef)); return *this; }
		
		//! can make itself look like an FSRef pointer, so it can be passed right into FileManager calls
		operator const FSRef* () const { return this; }
	};
											// Construction / destruction
	virtual								~CP_FileSystemSpec_Imp();

											// Getters
			void						GetParent( FSRef& inOutParentRef ) const;
			FSRefResult					GetParent() const;
			void						GetName( HFSUniStr255& inOutName ) const;

			CP_String					GetName() const;
			FSRefResult					GetFSRef() const;
			FSSpec						GetFSSpec() const;
			OSVolume					GetVolume() const;
			SInt16						GetVolumeID() const;
			SInt32						GetParID() const;
			
			CP_CFURL					GetAsCFURL();
			
			CP_String					GetFullPath_Posix() const									{ return GetFullPathInStyle(kCFURLPOSIXPathStyle); }
			CP_String					GetFullPath_HFS() const										{ return GetFullPathInStyle(kCFURLHFSPathStyle); }

											// operators
			bool						operator == ( const CP_FileSystemSpec_Imp& inFileSpec );
			bool						operator != ( const CP_FileSystemSpec_Imp& inFileSpec )		{ return( ! (*this == inFileSpec) ); }

											// Setters
			void						Set( FSRef inFileRef );
			void						Set(
												const CP_String& inName,
												FSRef inParentRef );
			void						Set(
												const FSSpec& inFSSpec,
												CFStringEncoding inEncoding = ::CFStringGetSystemEncoding() );
			void						Set(
												SInt16 vRefNum,
												SInt32 dirID );
			void						Set( const CP_String& inFullpath );
			void						SetName( const CP_String& inFileName );
			

											// Testers
										/*!
										*	Return true if this points to a valid file specification in the
										*	platforms native file system. The file can be valid, but not yet
										*	exist in the native file system.
										*	
										*	@return	bool const		True if the file specification is valid.
										*/
			bool						IsValid() const														{ return( fIsValid ); }

										/*!
										*	Return true if this points to a file that currently exists in the platforms
										*	native file system.
										*	
										*	@return	bool const		True if the file exists in the platforms native file system.
										*/
			bool						ExistsOnDisk() const												{ return( fExists ); }
			
			bool						IsDirectory() const;
			
			bool						IsEqual( const CP_FileSystemSpec_Imp& ) const;

			void						AssertDecomposedForm() const;

protected:
											// Construction / destruction
										CP_FileSystemSpec_Imp();
										CP_FileSystemSpec_Imp( const CP_FileSystemSpec_Imp& inFileSpec_Imp );
										CP_FileSystemSpec_Imp( const CP_String& inFullPath );
										CP_FileSystemSpec_Imp( FSRef inFileRef );
										CP_FileSystemSpec_Imp(
												const FSSpec& inFSSpec,
												CFStringEncoding inEncoding = ::CFStringGetSystemEncoding() );

											// operators
			CP_FileSystemSpec_Imp&		operator = ( const CP_FileSystemSpec_Imp& );

			FSRef						FSSpecToFSRef( const FSSpec& inFSSpec );
			FSSpec						FSRefToFSSpec( const FSRef& inFSRef );

			CP_String					GetFullPathInStyle(CFURLPathStyle inPathStyle) const;

	// class data
protected:
	bool								fExists;			/*! flag to indicate if the file exists in the platforms file system */
	bool								fIsValid;			/*! flag to indicate if this object holds a valid specification for a file,
																which may or may not currently exist in the file system. */
	FSRef								fRef;				/*! MacOS FSRef to the file */

	CP_String							fName;				/*! Name of the file */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC

#endif	// _H_CP_FileSystemSpec_Imp_Mac

