//=========================================================================
//
//	FILE:				CP_EventHandler_Imp_Win.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Mix-in class for classes that need to handle MacOS
//						CarbonEvents.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 162 $
//						Latest Checkout:  $Date: 2005-04-28 15:05:07 -0700 (Thu, 28 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_EventHandler_Imp_Win
#define _H_CP_EventHandler_Imp_Win
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_WIN32

	// CPLAT_Framework
#include "CP_Event.h"
#include "CP_New.h"
#include "CP_NoCopy.h"
#include "CP_OSStatus.h"
#include "CP_SignalSlot.h"
#include "CP_SignalSlot_Defines.h"
#include "CP_Types.h"

	// Standard Library
#include <map>

CPLAT_Begin_Namespace_CPLAT

//=========================================================================
//	CP_S_WinEventSpec
//=========================================================================
/*!
*	Event spec structure to hold an event class / event kind definition similiar
*	to the Carbon event structure.
*/
class CP_S_WinEventSpec
{
public:
	UInt32				fEventClass;
	UInt32				fEventKind;
	OSWindow			fHwnd;

								CP_S_WinEventSpec();
								CP_S_WinEventSpec(
										UInt32 inEventClass,
										UInt32 inEventKind,
										OSWindow inHwnd );
								CP_S_WinEventSpec( const CP_S_WinEventSpec& inEventSpec );

	CP_S_WinEventSpec&			operator =( const CP_S_WinEventSpec& inEventSpec );
	bool						operator ==( const CP_S_WinEventSpec& inEventSpec );
};

#if TARGET_CC_MSVC
	// MSVC C2678 bug workaround for VC++ v. 6	
	bool	operator <( const CP_S_WinEventSpec& inLHS, const CP_S_WinEventSpec& inRHS );
#endif

struct Less : public std::binary_function<CP_S_WinEventSpec, CP_S_WinEventSpec, bool>
{
	bool		operator()(const CP_S_WinEventSpec& x, const CP_S_WinEventSpec& y) const
					{
						return( (x.fEventClass < y.fEventClass) || 
							((x.fEventClass == y.fEventClass) && (x.fEventKind < y.fEventKind)));
					}
};

typedef std::multimap<CP_S_WinEventSpec, CP_EventSignal *, Less >					CP_EventHandler_Map;
typedef std::multimap<CP_S_WinEventSpec, CP_EventSignal *, Less >::iterator			CP_EventHandler_Iterator;

typedef std::pair<const CP_S_WinEventSpec, CP_EventSignal *>							CP_EventHandler_Entry;

class CP_EXPORT CP_EventHandler_Imp : public CP_SigSlot::CP_SlotObject, public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_EventHandler_Imp();
	//! Destructor.
	virtual								~CP_EventHandler_Imp();
	//@}
			
	//! @name Events
	//@{
			void						Add(
												UInt32 inClass, 
												UInt32 inKind, 
												CP_EventSignal *inEventSignal,
												OSWindow inWindow );

			void						Remove(
												UInt32 inEventClass,
												UInt32 inEventKind,
												OSWindow inWindow );
			void						RemoveAll();

			bool						HasHandler(
												UInt32 inEventClass,
												UInt32 inEventKind,
												OSWindow inWindow );

			CP_EventSignal				*FindHandler(
												UInt32 inEventClass,
												UInt32 inEventKind,
												OSWindow inWindow,
												LPARAM inLParam );
	//@}
	
	//! @name Getters
	//@{
										/*!
										*	Returns the Windows window proc callback used with this event handler.
										*	
										*	@exception	none
										*	
										*	@return		WNDPROC		Windows callback for this event handler.
										*/
			WNDPROC						GetWndProc()										{ return( fnOldWndProc ); }

										/*!
										*	Sets the Windows window proc callback used with this event handler.
										*
										*	@param		inWndProc	Windows callback to use for this event handler.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetWndProc( WNDPROC inWndProc )						{ fnOldWndProc = inWndProc; }

										/*!
										*	Returns the wParam for this event.
										*	
										*	@exception	none
										*	
										*	@return		WPARAM		The WPARAM for this event.
										*/
			WPARAM						GetWParam()											{ return( fWParam ); }

										/*!
										*	Returns the lParam for this event.
										*	
										*	@exception	none
										*	
										*	@return		LPARAM		The LPARAM for this event.
										*/
			LPARAM 						GetLParam()											{ return( fLParam ); }

			HWND						GetHWND()											{ return( fHWND ); }
	//@}

	//! @name Windows Messages
	//@{
	virtual	LRESULT CALLBACK 			EventHandlerProc(
												HWND inHwnd,
												UINT iMsg,
												WPARAM wParam,
												LPARAM lParam ) throw();

			LRESULT						CallDefault() throw();
	//@}

protected:
	virtual	OSWindow					FindOSWindowFromMessage(
												OSWindow inWindow,
												LPARAM inLParam,
												UINT inMsg );

private:

	// class data
protected:
	CP_EventHandler_Map					fEventHandlers;			/*! List of registered event handlers. */
	WPARAM								fWParam;				/*! WPARAM for this event. */
	LPARAM 								fLParam;				/*! LPARAM for this event. */

	WNDPROC								fnOldWndProc;			/*! Callback to be used when there is no event handler for this event.*/

	HWND								fHWND;
	UINT								fIMsg;
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_WIN32

#endif	// _H_CP_EventHandler_Imp_Win

